@@ -60,9 +60,9 @@ class EditorWindow(gtk.Window):
 
         # Create and fill the Effects menu
         effect_menu = gtk.Menu()
-        for name in controller.list_effects():
+        for name in app.list_effects():
             item = gtk.MenuItem(label=name)
-            item.connect('activate', self.effect, None, name)
+            item.connect('activate', self.effect, name)
             effect_menu.append(item)
         w = self.uimanager.get_widget('/menubar/Effects')
         w.set_submenu(effect_menu)
@@ -70,7 +70,7 @@ class EditorWindow(gtk.Window):
         self.waveform = GraphView(graph, selection, cursor)
         self.scrollbar = GraphScrollbar(graph)
         self.statusbar = gtk.Statusbar()
-        self.filedialog = FileDialog(controller.list_extensions())
+        self.filedialog = FileDialog(app.list_extensions())
 
         self.vbox = gtk.VBox()
         self.vbox.pack_start(self.menubar, expand=False, fill=False)
@@ -87,6 +87,12 @@ class EditorWindow(gtk.Window):
                            gtk.gdk.ACTION_COPY)
         self.connect("drag_data_received", self._open_dropped_files)
 
+        # Keyboard shortcuts
+        kval = gtk.gdk.keyval_from_name
+        self.handlers = {kval('space'): self.ctrl.toggle_play,
+                         kval('ISO_Level3_Shift'): self.ctrl.play}
+        self.connect('key_press_event', self.on_key_press_event)
+
         self.connect("delete-event", self.close)
         self._filename_update()
         self.ctrl.filename_changed.connect(self._filename_update)
@@ -140,14 +146,12 @@ class EditorWindow(gtk.Window):
               <toolitem action="ZoomIn"/>
               <separator/>
               <toolitem action="Start"/>
-              <toolitem action="Pause"/>
+              <toolitem action="Stop"/>
               <toolitem action="Play"/>
-              <toolitem action="End"/>
               <separator/>
               <toolitem action="Cut"/>
               <toolitem action="Copy"/>
               <toolitem action="Paste"/>
-              <toolitem action="Mix"/>
               <separator/>
               <toolitem action="Undo"/>
               <toolitem action="Redo"/>
@@ -155,7 +159,7 @@ class EditorWindow(gtk.Window):
             <accelerator action="Start"/>
             <accelerator action="End"/>
             <accelerator action="Play"/>
-            <accelerator action="Pause"/>
+            <accelerator action="Stop"/>
         </ui>'''
 
         uimanager = gtk.UIManager()
@@ -171,7 +175,7 @@ class EditorWindow(gtk.Window):
         actions = [('File', None, '_File'),
                    ('Edit', None, '_Edit'),
                    ('View', None, '_View'),
-                   ('Effects', None, '_Effects'),
+                   ('Effects', None, 'Effe_cts'),
                    ('Scalpel', None, '_Scalpel'),
                    ('New', gtk.STOCK_NEW, None, None, '', self.new),
                    ('Open', gtk.STOCK_OPEN, None, None, '', self.open),
@@ -182,7 +186,7 @@ class EditorWindow(gtk.Window):
                    ('Close', gtk.STOCK_CLOSE, None, None, '', self.close),
                    ('Quit', gtk.STOCK_QUIT, None, None, '', self.quit),
                    ('Play', gtk.STOCK_MEDIA_PLAY, None, 'p', '', self.play),
-                   ('Pause', gtk.STOCK_MEDIA_PAUSE, None, 'o', '', self.pause),
+                   ('Stop', gtk.STOCK_MEDIA_STOP, None, 'o', '', self.stop),
                    ('Start', gtk.STOCK_MEDIA_PREVIOUS, None, 'Home', '',
                                                               self.goto_start),
                    ('End', gtk.STOCK_MEDIA_NEXT, None, 'End', '',
@@ -232,6 +236,7 @@ class EditorWindow(gtk.Window):
         dialog = gtk.MessageDialog(parent=self, type=gtk.MESSAGE_WARNING)
         name = self.ctrl.filename() or "sound"
         name = os.path.basename(name)
+        name = name.replace('&', '&amp;')
         dialog.set_markup("<b>Save %s before closing?</b>" % name)
         dialog.add_button("Close _without saving", gtk.RESPONSE_NO)
         dialog.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
@@ -262,17 +267,33 @@ class EditorWindow(gtk.Window):
                             selection, targetType, time):
         """Open files dragged and dropped on the window."""
         filenames = selection.data.split()
+        prefix = 'file://'
         for filename in filenames:
             # Extract the actual filename
-            prefix = 'file://'
             if filename.startswith(prefix):
                 filename = filename[len(prefix):]
             filename = urllib.unquote(filename)
-            self.ctrl.open(filename)
+            self._do_open(filename)
 
     def display_error(self, title, text):
         display_error(title, text, parent=self)
 
+    def busy(method):
+        """A decorator to show a "busy" mouse cursor."""
+        def decorated(self, *args):
+            self.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
+            while gtk.events_pending():
+                gtk.main_iteration(False)
+            try:
+                method(self, *args)
+            finally:
+                self.window.set_cursor(None)
+        return decorated
+
+    @busy
+    def _do_open(self, filename):
+        self.ctrl.open(filename)
+
     # -- GTK Callbacks
 
     def __getattr__(self, name):
@@ -282,11 +303,10 @@ class EditorWindow(gtk.Window):
         not be passed to the controller method.
         
         """
-        if name in ["new", "save", "play", "pause",
+        if name in ["new", "save", "play", "stop",
                     "goto_start", "goto_end", "select_all",
                     "cut", "copy", "paste", "mix", "undo", "redo",
-                    "zoom_in", "zoom_out", "zoom_fit",
-                    "effect"]:
+                    "zoom_in", "zoom_out", "zoom_fit"]:
             method = getattr(self.ctrl, name)
             def forward(self, *args):
                 method(*args[1:])
@@ -294,6 +314,14 @@ class EditorWindow(gtk.Window):
         else:
             raise AttributeError(name)
 
+    def on_key_press_event(self, widget, event):
+        if event.keyval in self.handlers:
+            handler = self.handlers[event.keyval]
+            handler()
+    
+    @busy
+    def effect(self, widget, *args):
+        self.ctrl.effect(*args)
 
     def about(self, *args):
         d = gtk.AboutDialog()
@@ -306,11 +334,11 @@ class EditorWindow(gtk.Window):
         d.set_comments("A sound editor")
         d.run()
         d.destroy()
-        
+
     def open(self, *args):
         filename = self.filedialog.get_filename(action='open')
         if filename != None:
-            self.ctrl.open(filename)
+            self._do_open(filename)
 
     def save_as(self, *args):
         filename = self.filedialog.get_filename(action='save')
