@@ -16,8 +16,9 @@ class CairoWidget(gtk.DrawingArea):
     __gsignals__ = {"expose-event": "override"}
 
     def __init__(self):
-        super(CairoWidget, self).__init__()
-    
+        gtk.DrawingArea.__init__(self)
+        self._redrawing = False
+
     def do_expose_event(self, event):
         context = self.window.cairo_create()
         context.rectangle(event.area.x, event.area.y,
@@ -25,11 +26,14 @@ class CairoWidget(gtk.DrawingArea):
         context.clip()
         width, height = self.window.get_size()
         self.draw(context, width, height)
+        self._redrawing = False
 
     def redraw(self):
         # queue_draw() emits an expose event. Double buffering is used
         # automatically in the expose event handler.
-        self.queue_draw()
+        if not self._redrawing:
+            self._redrawing = True
+            self.queue_draw()
 
     def draw(self, context, width, height):
         """Must be overriden to draw to the cairo context."""
@@ -50,7 +54,7 @@ class LayeredCairoWidget(CairoWidget):
         
     def draw(self, context, width, height):
         for layer in self.layers:
-            layer.draw(context, width, height)
+            layer.stack(context, width, height)
 
 
 class LayeredGraphView(LayeredCairoWidget):
@@ -78,14 +82,12 @@ class LayeredGraphView(LayeredCairoWidget):
 class GraphView(LayeredGraphView):
     """Sound visualization widget for the main window.
 
-    * Three graphical layers: waveform, selection, and cursor.
-    * Mouse event listeners act on models (scroll and selection).
+    * Four graphical layers: background, waveform, selection, and cursor.
+    * Mouse event listeners act on models (scroll, selection, middle-click).
 
     """
     def __init__(self, graph, selection, cursor):
         super(GraphView, self).__init__(graph)
-        self._graph = graph
-        self._selection = selection
         self.layers.append(BackgroundLayer(self, selection))
         self.layers.append(WaveformLayer(self, graph))
         self.layers.append(SelectionLayer(self, selection))
@@ -98,181 +100,155 @@ class GraphView(LayeredGraphView):
 
 # -- Layers that can be added to LayeredGraphview.
 #
-class WaveformLayer(object):
+
+class Layer(object):
+    """The base class for layers.
+
+    Implements the cairo surface caching.
+
+    """
+    def __init__(self, layered):
+        self._layered = layered
+        self._must_draw = True
+        self._surface = None
+        layered.connect("size_allocate", self.resized)
+
+    def resized(self, widget, rect):
+        self._surface = None
+        self._must_draw = True
+
+    def update(self):
+        self._must_draw = True
+        self._layered.redraw()
+
+    def stack(self, context, width, height):
+        """ 
+        Paint the layer on top of the passed context.
+        """
+        if self._surface is None:
+            surface = context.get_target()
+            self._surface = surface.create_similar(cairo.CONTENT_COLOR_ALPHA,
+                                                   width, height)
+        if self._must_draw:
+            # clear context
+            c = cairo.Context(self._surface)
+            c.set_source_rgba(0, 0, 0, 0)
+            c.set_operator(cairo.OPERATOR_SOURCE)
+            c.paint()
+
+            self.draw(c, width, height)
+            self._must_draw = False
+
+        context.set_source_surface(self._surface, 0, 0)
+        context.set_operator(cairo.OPERATOR_OVER)
+        context.paint()
+    
+    def draw(context, width, height):
+        raise NotImplemented
+
+
+class WaveformLayer(Layer):
     """A layer for LayeredGraphView.
 
-    It paints the graph (the waveform). The cairo surface is cached,
-    it is redrawn only when graph has changed.
+    It paints the graph (the waveform).
 
     """
     def __init__(self, layered, graph):
-        self._layered = layered
+        Layer.__init__(self, layered)
         self._graph = graph
-        self._cache = None
         self.wavecolor = 0.0, 0.47058823529411764, 1.0
         graph.changed.connect(self.update)
 
-    def update(self):
-        self._cache = None
-        self._layered.redraw()
-
-    def draw_channel(self, values, surface, width, height):
-        "Draw one sound channel on a cairo surface."
-        c = cairo.Context(surface)
-
+    def draw_channel(self, values, context, ystart, width, height):
         # Line at zero
-        c.set_line_width(1)
-        c.set_source_rgb(0.2, 0.2, 0.2)
-        c.move_to(0, round(height / 2) + 0.5)
-        c.line_to(width, round(height / 2) + 0.5)
-        c.stroke()
+        context.set_line_width(1)
+        context.set_source_rgb(0.2, 0.2, 0.2)
+        context.move_to(0, ystart + round(height / 2) + 0.5)
+        context.line_to(width, ystart + round(height / 2) + 0.5)
+        context.stroke()
 
         # Waveform
-        c.set_source_rgb(*self.wavecolor)
+        context.set_source_rgb(*self.wavecolor)
         for i, (mini, maxi) in enumerate(values):
             # -1 <= mini <= maxi <= 1
-            # 0 <= ymin <= ymax <= height - 1
+            # ystart <= ymin <= ymax <= ystart + height - 1
             x = i
-            ymin = round((-mini * 0.5 + 0.5) * (height - 1))
-            ymax = round((-maxi * 0.5 + 0.5) * (height - 1))
+            ymin = ystart + round((-mini * 0.5 + 0.5) * (height - 1))
+            ymax = ystart + round((-maxi * 0.5 + 0.5) * (height - 1))
             if ymin == ymax:
                 # Fill one pixel 
-                c.rectangle(x, ymin, 1, 1)
-                c.fill()
+                context.rectangle(x, ymin, 1, 1)
+                context.fill()
             else:
                 # Draw a line from min to max
-                c.move_to(x + 0.5, ymin)
-                c.line_to(x + 0.5, ymax)
-                c.stroke()
+                context.move_to(x + 0.5, ymin)
+                context.line_to(x + 0.5, ymax)
+                context.stroke()
         
     def draw(self, context, width, height):
-        "Draw all sound channels."
-        if not self._cache:
-            surface = context.get_target()
-            self._cache = surface.create_similar(cairo.CONTENT_COLOR_ALPHA,
-                                                 width, height)
-            c = cairo.Context(self._cache)
-            channels = self._graph.channels()
-            numchan = len(channels)
-            for i in range(numchan):
-                s = surface.create_similar(cairo.CONTENT_COLOR_ALPHA,
-                                           width, height / numchan)
-                self.draw_channel(channels[i], s, width, height / numchan)
-                c.set_source_surface(s, 0, (height / numchan) * i)
-                c.set_operator(cairo.OPERATOR_OVER)
-                c.paint()
-                
-        context.set_source_surface(self._cache, 0, 0)
-        context.set_operator(cairo.OPERATOR_OVER)
-        context.paint()
+        channels = self._graph.channels()
+        numchan = len(channels)
+        for i in range(numchan):
+            y = (height / numchan) * i
+            self.draw_channel(channels[i], context, y, width, height / numchan)
 
 
-class BackgroundLayer(object):
+class BackgroundLayer(Layer):
     """A layer for LayeredGraphView.
 
     """
     def __init__(self, layered, selection):
-        self._layered = layered
-        self._cache = None
+        Layer.__init__(self, layered)
         self._selection = selection
         self._selection.changed.connect(self.update)
 
-    def update(self):
-        self._cache = None
-        self._layered.redraw()
-
     def draw(self, context, width, height):
-        if not self._cache:
-            surface = context.get_target()
-            self._cache = surface.create_similar(cairo.CONTENT_COLOR,
-                                                 width, height)
-            c = cairo.Context(self._cache)
-
-            # Black background
-            c.set_source_rgb(0, 0, 0)
-            c.paint()
-
-            # Selection background
-            if self._selection.selected():
-                c.set_source_rgb(0, 0, 0.2)
-                start, end = self._selection.pixels()
-                c.rectangle(start, 0, end - start, height)
-                c.fill()
-            
-        context.set_source_surface(self._cache, 0, 0)
-        context.set_operator(cairo.OPERATOR_SOURCE)
+        # Black background
+        context.set_source_rgb(0, 0, 0)
         context.paint()
 
+        # Selection background
+        if self._selection.selected():
+            context.set_source_rgb(0, 0, 0.2)
+            start, end = self._selection.pixels()
+            context.rectangle(start, 0, end - start, height)
+            context.fill()
 
-class SelectionLayer(object):
+
+class SelectionLayer(Layer):
     """A layer for LayeredGraphView.
 
-    It highlights the selected area. The cairo surface is cached, it
-    is redrawn only when selection has changed.
+    It highlights the selected area.
 
     """
     def __init__(self, layered, selection):
-        self._layered = layered
-        self._cache = None
+        Layer.__init__(self, layered)
         self._selection = selection
         self._selection.changed.connect(self.update)
 
-    def update(self):
-        self._cache = None
-        self._layered.redraw()
-
     def draw(self, context, width, height):
-        if not self._cache:
-            surface = context.get_target()
-            self._cache = surface.create_similar(cairo.CONTENT_COLOR_ALPHA,
-                                                 width, height)
-            c = cairo.Context(self._cache)
+        if self._selection.selected():
+            start, end = self._selection.pixels()
+            context.set_source_rgba(0, 0, 0, 0.5)
+            context.rectangle(0, 0, start, height)
+            context.rectangle(end, 0, width - end, height)
+            context.fill()
 
-            if self._selection.selected():
-                start, end = self._selection.pixels()
-                # darken everything ...
-                c.set_source_rgba(0, 0, 0, 0.5)
-                c.paint()
-
-                # ... then clear selection
-                c.set_operator(cairo.OPERATOR_CLEAR)
-                c.rectangle(start, 0, end - start, height)
-                c.fill()
-            
-        context.set_source_surface(self._cache, 0, 0)
-        context.set_operator(cairo.OPERATOR_OVER)
-        context.paint()
-
-class CursorLayer(object):
+class CursorLayer(Layer):
 
     def __init__(self, layered, cursor):
-        self._layered = layered
-        self._cache = None
+        Layer.__init__(self, layered)
         self._cursor = cursor
         self._cursor.changed.connect(self.update)
-        self._height = None
         self.rgba = (1, 1, 1, 0.5)
 
-    def update(self):
-        self._layered.redraw() # called in all layers !
-
     def draw(self, context, width, height):
-        if height != self._height:
-            self._height = height
-            surface = context.get_target()
-            self._cache = surface.create_similar(cairo.CONTENT_COLOR_ALPHA,
-                                                 1, height)
-            c = cairo.Context(self._cache)
-            c.set_source_rgba(*self.rgba)
-            c.set_line_width(1)
-            c.move_to(0.5, 0)
-            c.line_to(0.5, height)
-            c.stroke()
-
         x = self._cursor.pixel()
-        context.set_source_surface(self._cache, x, 0)
-        context.set_operator(cairo.OPERATOR_OVER)
-        context.paint()
+        context.set_source_rgba(*self.rgba)
+        context.set_line_width(1)
+        context.move_to(x + 0.5, 0)
+        context.line_to(x + 0.5, height)
+        context.stroke()
 
 
 # -- Mouse event listeners that act on models.
@@ -386,7 +362,7 @@ class MouseMiddleClick(object):
             delta = self._xlast - x
             self._xlast = x
             start, _ = self.graph.view()
-            self.graph.move_to(start + delta * self.graph.density())
+            self.graph.move_to(start + delta * self.graph.density)
 
 
 class PointerStyle(object):
